@@ -2,6 +2,7 @@
 # pylint: disable=too-many-branches
 # pylint: disable=line-too-long
 # pylint: disable=bare-except
+# pylint: disable=unused-import
 """
     Contains the general methods for manipulating files.
 """
@@ -24,7 +25,7 @@ import traceback
 from threading import Thread
 import logging
 
-# import ftputil
+import ftputil
 from secure_delete import secure_delete
 import colemen_string_utils as strUtils
 import utils.dir as directory
@@ -41,6 +42,8 @@ import utils.file_convert as convert
 
 logger = logging.getLogger(__name__)
 
+PILLOW_SUPPORTED_IMAGE_EXTENSIONS = ["bmp","dds","dib","eps","gif","icns","ico","im","jpg","jpeg","jpeg 2000","msp","pcx","png","ppm","sgi","spider","tga","tiff","webp","xbm"]
+
 
 def decompress(file_obj, compression="gzip"):
     '''
@@ -437,6 +440,8 @@ def copy(src, dest=False, **kwargs):
         [`ftp`=None] {object}
             A reference to the ftputil object to use.
 
+        [`ftp_credentials`=None] {object}
+            If threading is True and it is an ftp copy, you must provide ftp credentials.
 
         Return {type}
         ----------------------
@@ -451,6 +456,7 @@ def copy(src, dest=False, **kwargs):
         `method_name`: copy
     '''
     ftp = obj.get_kwarg(["ftp"], None, None, **kwargs)
+    ftp_credentials = obj.get_kwarg(["ftp_credentials"], None, None, **kwargs)
     threading = obj.get_kwarg(["threading"], True, (bool), **kwargs)
     max_threads = obj.get_kwarg(["max_threads"], 15, (int), **kwargs)
     min_thread_threshold = obj.get_kwarg(["min thread threshold", "min files thread"], 100, (int), **kwargs)
@@ -473,7 +479,11 @@ def copy(src, dest=False, **kwargs):
 
     if threading is True:
         if len(copy_list) >= min_thread_threshold:
-            _thread_file_action(copy_list, action="copy", max_threads=max_threads, min_thread_threshold=min_thread_threshold)
+            if ftp_credentials is not None:
+                _thread_file_action(copy_list, action="copy", max_threads=max_threads, min_thread_threshold=min_thread_threshold, ftp=ftp, ftp_credentials=ftp_credentials)
+                return
+            else:
+                _thread_file_action(copy_list, action="copy", max_threads=max_threads, min_thread_threshold=min_thread_threshold)
             return
 
     _copy_files_from_array(copy_list, ftp)
@@ -592,6 +602,7 @@ def _thread_file_action(file_list, **kwargs):
     action = obj.get_kwarg(["action"], "copy", (str), **kwargs)
     shred = obj.get_kwarg(["shred"], False, (bool), **kwargs)
     ftp = obj.get_kwarg(["ftp"], None, None, **kwargs)
+    ftp_credentials = obj.get_kwarg(["ftp_credentials"], None, None, **kwargs)
 
     if len(file_list) <= min_thread_threshold:
         max_threads = 1
@@ -604,11 +615,21 @@ def _thread_file_action(file_list, **kwargs):
         if end_idx > len(file_list):
             end_idx = len(file_list)
         files_array = file_list[start_idx:end_idx]
+        
         if action == "copy":
-            threads.append(Thread(target=_copy_files_from_array, args=(files_array,)))
+            if ftp_credentials is not None:
+                ftp = ftputil.FTPHost(ftp_credentials['address'], ftp_credentials['user_name'], ftp_credentials['password'])
+                ftp.synchronize_times()
+                threads.append(Thread(target=_copy_files_from_array, args=(files_array,ftp)))
+                continue
+            threads.append(Thread(target=_copy_files_from_array, args=(files_array,ftp)))
         if action == "delete":
             threads.append(Thread(target=_delete_files_from_array, args=(files_array, shred, ftp)))
 
+    if action == "copy":
+        print(f"    Copying {len(file_list)} files.")
+    if action == "delete":
+        print(f"    Deleting {len(file_list)} files.")
     for thread in threads:
         thread.start()
     for thread in threads:
@@ -1040,8 +1061,11 @@ def get_files(search_path=False, **kwargs):
             [`ignore`=[]] {str|list}
                 A term or list or terms to ignore if the file path contains any of them.
 
-            [`extensions`=[]] {str|list}
-                An extension or list of extensions that the file must have.
+            [`extensions|ext|extension`=[]] {str|list}
+                An extension or list of extensions that the file must have.\n
+                Can have leading periods or not.\n
+                if equal to "images" it will automatically search for these extensions:
+                    bmp,dds,dib,eps,gif,icns,ico,im,jpg,jpeg,jpeg 2000,msp,pcx,png,ppm,sgi,spider,tga,tiff,webp,xbm
 
             [`threaded`=True] {bool}
                 if True, the process is multi-threaded,
@@ -1096,8 +1120,8 @@ def get_files(search_path=False, **kwargs):
         include = [include]
 
     extension_array = strUtils.format.extension(obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
-    if isinstance(extension_array, (str)):
-        extension_array = [extension_array]
+    extension_array = _gen_extension_array(extension_array)
+            
 
     ftp = obj.get_kwarg(["ftp"], None, None, **kwargs)
     if ftp is not None:
@@ -1129,7 +1153,7 @@ def get_files(search_path=False, **kwargs):
 
                     if len(extension_array) > 0:
                         # print(f"filtering extension_array")
-                        file_ext = strUtils.format.extension(file_data['extension'])
+                        file_ext = strUtils.format.extension(file['extension'])
                         if file_ext not in extension_array:
                             continue
 
@@ -1152,6 +1176,17 @@ def get_files(search_path=False, **kwargs):
         # file_array = path_files + file_array
     return file_array
 
+def _gen_extension_array(ext_array):
+    extension_array = []
+    if isinstance(ext_array, (str)):
+        extension_array = [ext_array]
+    for ext in extension_array:
+        if ext == "images":
+            extension_array = extension_array + PILLOW_SUPPORTED_IMAGE_EXTENSIONS
+        else:
+            file_ext = strUtils.format.extension(ext)
+            extension_array.append(file_ext)
+    return extension_array
 
 def get_files_ftp(search_path=False, **kwargs):
     '''
